@@ -11,11 +11,12 @@
 ####################################################################################################
 # START glmdm.linear FUNCTION
 ####################################################################################################
-glmdm.linear <- function(Y, X, family=linear, num.reps=1000, log=TRUE, a1=3, b1=2, d=0.25, MM=15,VV=30,...) {
+glmdm.linear <- function(Y, X, family=linear, num.reps=1000, a1=3, b1=2, d=0.25, MM=15,VV=30,...) {
 # SET QUANTITIES FROM DATA
-n <- nrow(X); K <- ncol(X)
-beta <- runif(K,-3,3)
-mu <- 0; tau <- 1; 
+#n <- nrow(X); K <- ncol(X)
+#beta <- runif(K,-3,3)
+#mu <- 0; tau <- 1; 
+n <- nrow(X)
 # For the prior of m, we use Gamma(a,b)
 # Here ab=MM and ab^2=VV. 
 Sh <- (MM^2)/VV
@@ -25,7 +26,7 @@ Sc <- VV/MM
 # RUN A GLM TO GET hat.beta and hat.var
 ####################################################################################################
 
-suppressWarnings(glm.out <- glm(Y ~ X,family=binomial(link=probit)))
+suppressWarnings(glm.out <- glm(Y ~ X))
 hat.beta <- coefficients(summary(glm.out))[,1]
 hat.var  <- (coefficients(summary(glm.out))[,2])^2
 
@@ -45,11 +46,9 @@ p             <- length(hat.beta)
 aa            <- chol(solve(t(X) %*% X + diag(1/sigma.beta.sq, nrow=ncol(X))))
 SIG           <- t(X)%*%X + diag(1/sigma.beta.sq, nrow=ncol(X))				# VAR/COVAR MATRIX
 SIG.inv       <- solve(SIG)								# INVERT HERE FOR EFFICIENCY
-Z             <- rep(NA,length=n)							# Z TO FILL-IN DURING LOOPING
 beta          <- rmvnorm(1, mean=hat.beta, sigma=diag(hat.var))				# STARTING VALUES FROM ABOVE
 beta          <- rbind( beta,matrix(rep(NA,num.reps*ncol(beta)),ncol=ncol(beta)) )	# MATRIX TO FILL IN DURING LOOP
-n.i           <- rep(1,n);   								# LOOP NEEDS REP OF n'S
-q             <- rdirichlet(1, alpha=n.i)						# PROBABILITY OF ASSIGNMENT
+q          <- c(rdirichlet(1, rep(1, n)))					# PROBABILITY OF ASSIGNMENT
 A.n 	      <- sample(1:n,n,replace=TRUE,q)				 		# CREATES THE ASSIGNMENTS.
 A.K 	      <- table(A.n)						 		# CREATES THE LIST OF OCCUPIED
 A.K.labels    <- as.numeric(names(A.K))  						# LOCATIONS OF OCCUPIED
@@ -62,13 +61,11 @@ psi           <- B[A.n] 								# MAP TABLE psi'S TO CASES
 like.K        <- 0									# LOG-LIKE STARTS AT ZERO
 X.beta1       <- X%*%beta[1,]								# MULTIPLY X AND beta IN ADVANCE
 for (i in 1:n)  
-    like.K <- like.K + Y[i] * pnorm( X.beta1[i] + psi[i],log=TRUE) + 
-		(1-Y[i]) * (1 - pnorm( X.beta1[i] + psi[i] )) + 
-		dnorm(psi[i], mean=0, sd=tau,log=TRUE)
+    like.K <- like.K + dnorm(Y[i], mean=X.beta1[i] + psi[i], log=TRUE) + dnorm(psi[i], mean=0, sd=tau,log=TRUE)
 like.K <- exp(like.K + sum(lgamma(A.K)))
 
 # SETUP TO SAVE GIBBS SAMPLER VALUES
-tau.sq.post <- psi.post <- beta.post <- xi.post <- K.save <- like.K.save <- m.save <- NULL 
+tau.sq.post <- psi.post <- xi.post <- K.save <- like.K.save <- m.save <- NULL 
 
 ####################################################################################################
 # NEW POSTERIOR FOR m FUNCTION
@@ -93,7 +90,7 @@ for (M in 1:num.reps)  {
     mult.old <- dmultinom(x=A.K, prob=q[A.K.labels])
     #print("CREATE NEW 'A' MATRIX, 'can' STANDS FOR CANDIDATE") 
     pq                <- nk +1   
-    new.q             <- rdirichlet(1, alpha=pq)
+    new.q             <- rdirichlet(1, pq)
     A.n.can           <- sample(1:n,n,replace=TRUE,new.q)                                           
     A.K.can           <- table(A.n.can)                                                           
     A.K.labels.can    <- as.numeric(names(A.K.can))                                             
@@ -106,7 +103,7 @@ for (M in 1:num.reps)  {
     like.K.can        <- 0                                                                      
     X.betaM           <- X%*%beta[M,]                                                          
     for (i in 1:n)
-	like.K.can    <- like.K.can - log(3^2)*n/2 - (1/(2*3^2))*(Y[i] - X.betaM[i] + psi.can[i])^2
+	like.K.can    <- like.K.can + dnorm(Y[i], mean=X.betaM[i] + psi.can[i], log=TRUE) + dnorm(psi.can[i], mean=0, sd=tau,log=TRUE)
         #like.K.can    <- like.K.can + Y[i] * pnorm( X.betaM[i] + psi.can[i],log=TRUE) +
         #                 (1-Y[i]) * (1 - pnorm( X.betaM[i] + psi.can[i] )) +
         #                 dnorm(psi.can[i], mean=0, sd=tau,log=TRUE)
@@ -134,19 +131,27 @@ for (M in 1:num.reps)  {
     }
 
     #print("UPDATE 'z': Truncated Normal")
-    for (j in 1:n){
-	mean = X[j,]%*%beta[M,] + psi[j]
-        Z[j] <- rtnorm(1, mean=mean, sd=1, lower=-Inf, upper=0)
-        if (Y[j]==1) Z[j] <- rtnorm(1, mean=mean, sd=1, lower=0, upper=Inf)
-    }
+#    for (j in 1:n){
+#	mean = X[j,]%*%beta[M,] + psi[j]
+#        Z[j] <- rtnorm(1, mean=mean, sd=1, lower=-Inf, upper=0)
+#        if (Y[j]==1) Z[j] <- rtnorm(1, mean=mean, sd=1, lower=0, upper=Inf)
+#    }
 
     #print("UPDATE 'beta', 'tau' AND 'eta'")
-    mn            <- SIG.inv %*% (t(X) %*% (Z-psi)) 
+    mn            <- SIG.inv %*% (t(X) %*% (Y-psi)) 
     Mb            <- t(aa) %*% array(rnorm(p), c(p, 1)) + mn 
     beta[M+1, ]   <- t(Mb) 
-    bb            <- diag( sqrt( sigma.sq*tau.sq/(A.K + sigma.sq) ) )
-    meta          <- (1/sigma.sq) * (bb^2) %*% (Z-X%*%beta[M+1,])[A.K] 
-	eta           <- t(bb) %*% array(rnorm(K), c(K, 1)) + meta 
+    
+    A.m1          <- matrix(0, nrow=n, ncol=n)
+    for (i in 1:n){ A.m1[i,A.n[i]] <- 1}
+    A.K.m         <- A.m1[,which(apply(A.m1,2,sum)>0)]
+    S.eta.inv     <- 1/sigma.sq *t(A.K.m) %*% A.K.m + diag(1/tau.sq, nrow=K)
+    S.eta         <- solve(S.eta.inv)
+    bb            <- chol(S.eta) 
+    meta          <- 1/sigma.sq * S.eta %*% t(A.K.m) %*% (Y-X%*%beta[M+1,])
+    eta           <- t(bb) %*% array(rnorm(K), c(K, 1)) + meta
+    psi           <- A.K.m %*% eta
+
     tau.sq        <- rinvgamma(1,shape=(K)/2+a1,scale=sum(eta^2)/2+b1)
     # If tau.sq is infinity; 1.7e+100 is reasonable big?!
     if(is.finite(tau.sq)==FALSE) tau.sq <- 1.7e+100
@@ -178,6 +183,18 @@ for (M in 1:num.reps)  {
     m.save <- c(m.save,m) 
    if (M %% 100 == 0) print(paste("finished iteration",M))
 }
-
+   beta.post <- apply(beta[(round(num.reps/2)+1):(num.reps+1),],2,mean)
+   beta.sd   <- apply(beta[(round(num.reps/2)+1):(num.reps+1),],2,sd)
+   Y.hat     <- c(X %*% beta.post)
+   resid     <- c(Y - Y.hat)
+   
+   K.est     <- c()
+for (l in 1:length(m.save)){
+   K.est[l] <- m.save[l]*sum(1/seq(m.save[l],m.save[l]+n-1,by=1))	}
+   K.post   <- mean(K.est[round(length(m.save)/2):length(m.save)])
+   K.sd     <- sd(K.est[round(length(m.save)/2):length(m.save)])
+   
+   out <- list(coefficients= beta.post, standard.error = beta.sd, fitted.values = Y.hat, residuals = resid, K.est = K.post, K.sd = K.sd)
+   out
 } # END TO glmdm FUNCTION CALL
 
